﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace FpgBinExtract
{
	// fun note: this program also works with Fire Pro Wrestling D's SBL.BIN
	class Program
	{
		public static uint GetUint32(byte[] data, int startPos) {
			return (uint)((data[startPos + 3] << 24) | (data[startPos + 2] << 16) | (data[startPos + 1] << 8) | data[startPos]);
		}
	
		static void Main(string[] args) {
			string filePath = "";

			Console.WriteLine("Fire Pro Wrestling G FPG.BIN Extractor");
			if (args.Length == 0) {
				Console.WriteLine("(No file passed in, assuming FPG.BIN is in the same directory)");
				filePath = "./FPG.BIN";
			}
			else {
				filePath = args[0];
			}

			// get the data from the file
			byte[] fpgbinData;
			Console.WriteLine("Loading file...");
			using(FileStream fs = new FileStream(filePath,FileMode.Open)){
				fpgbinData = new byte[fs.Length];
				using(BinaryReader br = new BinaryReader(fs)){
					br.Read(fpgbinData, 0, (int)fs.Length);
					br.Close();
					fs.Close();
				}
			}
			Console.WriteLine("Loading complete.");

			// lists!
			uint[] fileLocations = new uint[512];
			uint[] fileSizes = new uint[512];

			// data is in the byte array; look at the first 1024 bytes for the file listing
			for (int i = 0; i < 512; i++) {
				fileLocations[i] = GetUint32(fpgbinData, i * 8);
				fileSizes[i] = GetUint32(fpgbinData, (i * 8) + 4);
			}

			Console.WriteLine("Begin exporting...");
			// perform exporting
			for (int i = 0; i < 512; i++) {
				if (fileSizes[i] > 0) {
					FileStream fs = new FileStream(String.Format("{0:X8}.bin", fileLocations[i]), FileMode.Create);
					BinaryWriter bw = new BinaryWriter(fs);
					byte[] outData = new byte[fileSizes[i]];
					Array.Copy(fpgbinData, fileLocations[i], outData, 0, fileSizes[i]);
					bw.Write(outData);
					bw.Close();
					fs.Close();
				}
			}
			Console.WriteLine("Exporting finished.");
		}
	}
}
